package com.tencent.flutter_integration;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.HashMap;

import androidx.annotation.NonNull;
import io.flutter.embedding.android.FlutterFragment;
import io.flutter.embedding.android.FlutterView;
import io.flutter.embedding.engine.FlutterEngine;

public class FlutterIntegrationFragment extends FlutterFragment {

    public String routeName;
    public Serializable params;

    private static final String KEY_ROUTE_NAME = "FlutterIntegrationFragment_KEY_ROUTE_NAME";
    private static final String KEY_PARAMS = "FlutterIntegrationFragment_KEY_PARAMS";
    private static final String KEY_PAGEID = "FlutterIntegrationFragment_KEY_PAGEID";

    private String mPageId;
    private FlutterView mFlutterView;
    private ScreenBroadcastReceiver mScreenReceiver;

    public FlutterIntegrationFragment() {
        super();
    }

    public static FlutterIntegrationFragment newInstance(String routeName) {
        return newInstance(routeName, null);
    }

    public static FlutterIntegrationFragment newInstance(String routeName, Serializable params) {
        FlutterIntegrationFragment fragment =
                new NewEngineFragmentBuilder(FlutterIntegrationFragment.class).initialRoute("/").transparencyMode(FlutterView.TransparencyMode.opaque)
                        .build();
        fragment.routeName = routeName;
        fragment.params = params;

        return fragment;
    }

    @Override
    public FlutterEngine provideFlutterEngine(Context context) {
        return FlutterIntegrationManager.sharedInstance().container.flutterEngine;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View splashView = super.onCreateView(inflater, container, savedInstanceState);
        if ((splashView instanceof FrameLayout)) {
            mFlutterView = (FlutterView) ((FrameLayout) splashView).getChildAt(0);
        }
        if (mScreenReceiver == null) {
            mScreenReceiver = new ScreenBroadcastReceiver();
            mScreenReceiver.registerScreenBroadcastReceiver(getActivity());
        }
        return splashView;
    }

    @Override
    public void onStart() {
        super.onStart();
        Log.e("FlutterIntegration", String.format("onStart, pageId:%s", mPageId));
    }

    @Override
    public void onResume() {
        super.onResume();

        FlutterIntegrationManager.sharedInstance().container.fragment = this;
        mFlutterView.attachToFlutterEngine(FlutterIntegrationManager.sharedInstance().container.flutterEngine);
        openPage();
        Log.e("FlutterIntegration", String.format("onResume, pageId:%s", mPageId));
    }

    @Override
    public void onPause() {
        super.onPause();
        Log.e("FlutterIntegration", String.format("onPause, pageId:%s", mPageId));
    }

    @Override
    public void onStop() {
        super.onStop();

        if (FlutterIntegrationManager.sharedInstance().container.fragment != null &&
                FlutterIntegrationManager.sharedInstance().container.fragment.mPageId.equals(this.mPageId)) {
            FlutterIntegrationManager.sharedInstance().container.fragment = null;

        } else if (FlutterIntegrationManager.sharedInstance().container.fragment != null) {
            FlutterIntegrationManager.sharedInstance().container.fragment.onResume();
        }
        Log.e("FlutterIntegration", String.format("onStop, pageId:%s", mPageId));
    }

    @Override
    public void onDetach() {
        super.onDetach();

        if (FlutterIntegrationManager.sharedInstance().container.fragment != null &&
                FlutterIntegrationManager.sharedInstance().container.fragment.mPageId.equals(this.mPageId)) {
            FlutterIntegrationManager.sharedInstance().container.fragment = null;

        } else if (FlutterIntegrationManager.sharedInstance().container.fragment != null) {
            FlutterIntegrationManager.sharedInstance().container.fragment.onResume();
        }
        Log.e("FlutterIntegration", String.format("onDetach, pageId:%s", mPageId));
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        removePage();
        mScreenReceiver.unRegisterScreenBroadcastReceiver(getActivity());
        Log.e("FlutterIntegration", String.format("onDestroyView, pageId:%s", mPageId));
    }

    @Override
    public void onSaveInstanceState(Bundle outState) {
        super.onSaveInstanceState(outState);
        outState.putString(KEY_ROUTE_NAME, routeName);
        outState.putSerializable(KEY_PARAMS, params);
        outState.putString(KEY_PAGEID, mPageId);
    }

    @Override
    public void onActivityCreated(Bundle savedInstanceState) {
        super.onActivityCreated(savedInstanceState);
        if (savedInstanceState != null) {
            routeName = savedInstanceState.getString(KEY_ROUTE_NAME);
            mPageId = savedInstanceState.getString(KEY_PAGEID);
            params = savedInstanceState.getSerializable(KEY_PARAMS);
        }
    }

    @Override
    public boolean shouldDestroyEngineWithHost() {
        return false;
    }

    @Override
    public boolean shouldAttachEngineToActivity() {
        return false;
    }

    @SuppressLint("DefaultLocale")
    private String getPageId() {
        if (mPageId == null || mPageId.length() <= 0) {
            mPageId = String.format("%s_%d", routeName, System.currentTimeMillis());
        }
        return mPageId;
    }

    private void openPage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("routeName", routeName);
        map.put("pageId", getPageId());
        if (params != null) {
            map.put("params", params);
        }

        FlutterIntegrationManager.sharedInstance().container.methodChannel.invokeMethod("openPage", map);
        Log.e("FlutterIntegration", String.format("openPage, pageId:%s", mPageId));
    }

    private void removePage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("pageId", mPageId);

        FlutterIntegrationManager.sharedInstance().container.methodChannel.invokeMethod("removePage", map);
        Log.e("FlutterIntegration", String.format("removePage, pageId:%s", mPageId));
    }

    @NonNull
    @Override
    public String toString() {
        return super.toString() + ", pageId:" + getPageId();
    }
}
